#include "Status.h"
#include "Vram.h"
#include "Main.h"
#include "Sprite.h"
#include "Print.h"
#include "Chars.h"

constexpr byte Char_Remain = Char_Man;

static ptr<byte> PrintS(ptr<byte> pVram, ptr<byte> p)
{
    byte c;
    while ((c = *p) != 0) {
        pVram = PrintC(pVram, c);
        ++p;
    }
    return pVram;
}

void PrintStatus() 
{
    PrintS(VramFrontOrg + 1 * VramStep, "SCORE");
    PrintS(VramFrontOrg + 7 * VramStep, "STAGE");
    PrintByteNumber2(VramFrontOrg + VramRowSize + 10 * VramStep, CurrentStage + 1);
    PrintS(VramFrontOrg + 13 * VramStep, "TIME");
    {
        ptr<byte> pVram;
        pVram = VramFrontOrg + 20 * VramStep;
        if (RemainCount > 1) {
            byte i;
            i = RemainCount - 1;
            if (i > 2) {
                Put2C(pVram, Char_Remain);
                pVram += 2 * VramStep;
                pVram = Put(pVram, Char_Space);
                pVram = Put(pVram, Char_Space);
                pVram += VramRowSize - 2 * VramStep;
                pVram = Put(pVram, i + 0x10);
                pVram = Put(pVram, Char_Space);
            }
            else {
                do {
                    Put2C(pVram, Char_Remain);
                    pVram += 2 * VramStep;
                    --i;
                } while (i > 0);
            }
        }
    }
    PrintScore();
    PrintTime();
}

void PrintScore()
{
    ptr<byte> pVram;
    pVram = PrintNumber5(VramFrontOrg + VramRowSize, Score);
    PrintC(pVram, '0');
}

void PrintTime()
{
    PrintByteNumber3(VramFrontOrg + VramRowSize + 14 * VramStep, StageTime);
}

void PrintGameOver()
{
    PrintS(VramFrontOrg + VramRowSize * 7 + (24 - 9) / 2 * VramStep, "GAME OVER");
}

void PrintTimeUp() 
{
    PrintS(VramFrontOrg + VramRowSize * 7 + (24 - 7) / 2 * VramStep, "TIME UP");
}

void Title()
{
    ClearScreen(); 
    HideAllSprites();
    // UpdateSprites();
    PrintStatus();
    {
        static const byte[] TitleBytes = {
            //	C
            0x40, 0x4e, 0x45, 0x4b, 
            0x4c, 0x43, 0x40, 0x40, 
            0x44, 0x4b, 0x40, 0x4a, 
            0x40, 0x44, 0x45, 0x41, 
            //	R
            0x4c, 0x47, 0x45, 0x4b, 
            0x4c, 0x43, 0x48, 0x4f, 
            0x4c, 0x47, 0x4f, 0x42, 
            0x44, 0x41, 0x44, 0x45, 
            //	A
            0x40, 0x4e, 0x4d, 0x42, 
            0x4c, 0x43, 0x40, 0x4f, 
            0x4c, 0x47, 0x45, 0x4f, 
            0x44, 0x41, 0x40, 0x45, 
            //	C
            0x40, 0x4e, 0x45, 0x4b, 
            0x4c, 0x43, 0x40, 0x40, 
            0x44, 0x4b, 0x40, 0x4a, 
            0x40, 0x44, 0x45, 0x41, 
            //	K
            0x4c, 0x43, 0x48, 0x47, 
            0x4c, 0x4b, 0x47, 0x40, 
            0x4c, 0x43, 0x4d, 0x42, 
            0x44, 0x41, 0x40, 0x45, 
            //	Y
            0x4c, 0x43, 0x4c, 0x43, 
            0x44, 0x4b, 0x4e, 0x41, 
            0x40, 0x4c, 0x43, 0x40, 
            0x40, 0x44, 0x41, 0x40, 
        };
        constexpr byte LogoLength = 6;
        ptr<byte> p;
        ptr<byte> pVram;
        pVram = VramFrontOrg + VramRowSize * 6 + (VramWidth - 4 * LogoLength) / 2 * VramStep;
        p = TitleBytes;
        repeat (LogoLength) {
            repeat (4) {
                repeat (4) {
                    pVram = Put(pVram, *p);
                    ++p;
                }
                pVram += VramRowSize - 4 * VramStep;
            }
            pVram += 4 * VramStep - VramRowSize * 4;
        }
        PrintS(VramFrontOrg + VramRowSize * 10 + (VramWidth - 4 * LogoLength - 4) * VramStep, "MINI");
    }
    PrintS(VramFrontOrg + VramRowSize * 13 + 3 * VramStep, "PUSH START BUTTON");
    PrintS(VramFrontOrg + VramRowSize * 15 + 12 * VramStep, "INUFUTO 2023");
    // {
    //     Movable m;
    //     m.x = 8*6;
    //     m.y = 8*3;
    //     m.sprite = 0;
    //     ShowSprite(&m, Char_Man);
    //     UpdateSprites();
    // }
   
    // {
    //     byte c;
    //     word v;
    //     v = Vram+32*8*2;
    //     c = 0;
    //     repeat (0) {
    //         v = Put(v, c);
    //         ++c;
    //     }
    // }
}